/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#ifndef EXTENSIONMIDI_GLOBALKEYBOARD_H
#define EXTENSIONMIDI_GLOBALKEYBOARD_H

#include "extension_midi.h"

class ExtensionMidi_GlobalKeyboard_Configuration;
class ExtensionMidi_GlobalKeyboard_Gui;

class ExtensionMidi_GlobalKeyboard: public ExtensionMidi
{
    Q_OBJECT

public:
    ExtensionMidi_GlobalKeyboard();
    ~ExtensionMidi_GlobalKeyboard();

    // Identification
    QString getTitle() const override { return "Global keyboards"; }
    QString getIdentifier() const override { return "globalKeyboards"; }
    QString getIconPath() const override { return ":/icons/keyboards.svg"; }

    /// Priority for processing MIDI signals
    int getMidiPriority() const override { return 50; }

    // MIDI signals
    bool processKey(int channel, int key, int vel) override;
    bool processPolyPressureChanged(int channel, int key, int pressure) override;
    bool processMonoPressureChanged(int channel, int value) override;
    bool processControllerChanged(int channel, int num, int value) override;
    bool processBendChanged(int channel, float value) override;
    bool processBendSensitivityChanged(int channel, float semitones) override;

protected:
    // Interface
    QWidget * getGui() override;

private slots:
    void onMuteChannel(quint8 channel);
    void onPresetActivated(quint8 channel, quint8 sf2Id, quint8 presetId, quint8 currentState, quint8 activatedState);
    void onPresetDeactivated(quint8 channel, quint8 sf2Id, quint8 presetId, quint8 currentState, quint8 deactivatedState);

private:
    void processKeyInternal(int channel, int key, int vel);

    ExtensionMidi_GlobalKeyboard_Configuration * _extensionConfig;
    ExtensionMidi_GlobalKeyboard_Gui * _gui;
    quint8 ** _triggeredKeys;
};

#endif // EXTENSIONMIDI_GLOBALKEYBOARD_H
