/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#ifndef EXTENSIONMIDI_GLOBALKEYBOARD_CONFIGURATION_H
#define EXTENSIONMIDI_GLOBALKEYBOARD_CONFIGURATION_H

#include <QList>
#include <QVariant>

class ActivatedPreset
{
public:
    ActivatedPreset(quint8 sf2Id, quint8 _presetId, quint8 state) :
        _sf2Id(sf2Id),
        _presetId(_presetId),
        _state(state)
    {}

    quint8 _sf2Id;
    quint8 _presetId;
    quint8 _state; // combinaison of: 0x01 => normal octave, 0x02 => super octave, 0x04 => sub octave
};

class ExtensionMidi_GlobalKeyboard_Configuration: public QObject
{
    Q_OBJECT

public:
    ExtensionMidi_GlobalKeyboard_Configuration(QString extensionIdentifier);
    ~ExtensionMidi_GlobalKeyboard_Configuration();

    // Channels
    void addChannel(int channel);
    void removeChannel(int channel);
    void setChannels(QList<int> channels);

    // Soundfont
    void onSoundfontClosed(int sf2Index);

    int _channelByKeyboard[16];
    bool _usedChannels[16];

    // Activated presets
    quint8 getActivationState(quint8 channel, quint8 sf2Id, quint8 presetId);
    void setActivationState(quint8 channel, quint8 sf2Id, quint8 presetId, quint8 state);
    QVector<ActivatedPreset> _presetPerChannel[16];

    // Other
    QString _extensionIdentifier;

signals:
    void muteChannel(quint8 channel);
    void presetActivated(quint8 channel, quint8 sf2Id, quint8 presetId, quint8 currentState, quint8 activatedState);
    void presetDeactivated(quint8 channel, quint8 sf2Id, quint8 presetId, quint8 currentState, quint8 deactivatedState);

private:
    void loadChannelsFromConfig();
    void saveChannelsToConfig();
};

#endif // EXTENSIONMIDI_GLOBALKEYBOARD_CONFIGURATION_H
