/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "extensionmidi_globalkeyboard_keyboardcell.h"
#include "ui_extensionmidi_globalkeyboard_keyboardcell.h"
#include "contextmanager.h"

ExtensionMidi_GlobalKeyboard_KeyboardCell::ExtensionMidi_GlobalKeyboard_KeyboardCell(int channel, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ExtensionMidi_GlobalKeyboard_KeyboardCell)
{
    ui->setupUi(this);
    ui->pushRemove->setIcon(ContextManager::theme()->getColoredSvg(":/icons/minus.svg", QSize(16, 16), ThemeManager::BUTTON_TEXT));
    connect(ui->channelSelector, SIGNAL(midiChannelChanged(int,bool)), this, SLOT(onMidiChannelChanged(int,bool)));

    // Keyboard style
    if (ContextManager::theme()->isDark(ThemeManager::LIST_BACKGROUND, ThemeManager::LIST_TEXT))
    {
        ui->keyboard->set(PianoKeybd::PROPERTY_COLOR_BLACK_KEYS, ContextManager::theme()->getColor(ThemeManager::LIST_BACKGROUND));
        ui->keyboard->set(PianoKeybd::PROPERTY_COLOR_WHITE_KEYS, ContextManager::theme()->getColor(ThemeManager::LIST_TEXT));
    }
    else
    {
        ui->keyboard->set(PianoKeybd::PROPERTY_COLOR_BLACK_KEYS, ContextManager::theme()->getColor(ThemeManager::LIST_TEXT));
        ui->keyboard->set(PianoKeybd::PROPERTY_COLOR_WHITE_KEYS, ContextManager::theme()->getColor(ThemeManager::LIST_BACKGROUND));
    }

    // Current channel
    if (channel != -1)
        ui->channelSelector->setMidiChannel(channel);

    ui->keyboard->set(PianoKeybd::PROPERTY_COLOR_1, ContextManager::theme()->getColor(ThemeManager::HIGHLIGHTED_BACKGROUND));
    ui->keyboard->setFrameStyle(0);
    ui->keyboard->set(PianoKeybd::PROPERTY_KEY_MIN, 36);
    ui->keyboard->set(PianoKeybd::PROPERTY_KEY_NUMBER, 61);
    ui->keyboard->set(PianoKeybd::PROPERTY_CHANNEL, ui->channelSelector->getMidiChannel());
}

ExtensionMidi_GlobalKeyboard_KeyboardCell::~ExtensionMidi_GlobalKeyboard_KeyboardCell()
{
    delete ui;
}

void ExtensionMidi_GlobalKeyboard_KeyboardCell::disableRemoveButton()
{
    ui->pushRemove->setEnabled(false);
    ui->pushRemove->hide();
}

void ExtensionMidi_GlobalKeyboard_KeyboardCell::on_pushRemove_clicked()
{
    emit(pushRemoveClicked());
}

void ExtensionMidi_GlobalKeyboard_KeyboardCell::onMidiChannelChanged(int midiChannel, bool keepInternal)
{
    ui->keyboard->set(PianoKeybd::PROPERTY_CHANNEL, midiChannel);
    if (!keepInternal)
        emit(channelChanged());
}

int ExtensionMidi_GlobalKeyboard_KeyboardCell::getChannel()
{
    return ui->channelSelector->getMidiChannel();
}

int ExtensionMidi_GlobalKeyboard_KeyboardCell::heightForWidth(int width) const
{
    int margin = ui->gridLayout->contentsMargins().bottom();
    int spacing = ui->gridLayout->verticalSpacing();
    return ui->pushRemove->height() + spacing + ui->keyboard->heightForWidth(width - 2 * margin) + 2 * margin;
}

void ExtensionMidi_GlobalKeyboard_KeyboardCell::inputNote(int key, int vel)
{
    if (vel > 0)
        ui->keyboard->inputNoteOn(key, vel);
    else
        ui->keyboard->inputNoteOff(key);
}
