/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "extensionmidi_globalkeyboard_presetcell.h"
#include "ui_extensionmidi_globalkeyboard_presetcell.h"
#include "midi_global_keyboard/extensionmidi_globalkeyboard_configuration.h"
#include "contextmanager.h"

ExtensionMidi_GlobalKeyboard_PresetCell::ExtensionMidi_GlobalKeyboard_PresetCell(ExtensionMidi_GlobalKeyboard_Configuration * extensionConfig,
                                                                                 int channel, int indexSf2, int indexElt, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ExtensionMidi_GlobalKeyboard_PresetCell),
    _extensionConfig(extensionConfig),
    _channel(channel),
    _indexSf2(indexSf2),
    _indexElt(indexElt)
{
    ui->setupUi(this);

    // Style
    QString color = ContextManager::theme()->mix(ContextManager::theme()->getColor(ThemeManager::LIST_BACKGROUND),
                                                 ContextManager::theme()->getColor(ThemeManager::LIST_TEXT), 0.5).name();
    QString checkBoxStyle = QString(
                "QCheckBox::indicator:unchecked { border: 1px solid %1; width: %3px; height: %3px; background: none; } \
                QCheckBox::indicator:checked { border: 1px solid %2; width: %3px; height: %3px; background: %2; }")
                .arg(color)
                .arg(ContextManager::theme()->getColor(ThemeManager::HIGHLIGHTED_BACKGROUND).name())
                .arg(this->height());
    ui->check16->setStyleSheet(checkBoxStyle);
    ui->check8->setStyleSheet(checkBoxStyle);
    ui->check4->setStyleSheet(checkBoxStyle);

    ui->labelSub->setStyleSheet(QString("QLabel { color: %1; margin-right: 2px; }").arg(color));
    ui->labelSuper->setStyleSheet(QString("QLabel { color: %1; }").arg(color));

    // Initialize the checkboxes
    quint8 state = _extensionConfig->getActivationState(channel, indexSf2, indexElt);
    if (state & 0x01)
    {
        ui->check8->blockSignals(true);
        ui->check8->setChecked(true);
        ui->check8->blockSignals(false);
    }
    if (state & 0x02)
    {
        ui->check4->blockSignals(true);
        ui->check4->setChecked(true);
        ui->check4->blockSignals(false);
    }
    if (state & 0x04)
    {
        ui->check16->blockSignals(true);
        ui->check16->setChecked(true);
        ui->check16->blockSignals(false);
    }
}

ExtensionMidi_GlobalKeyboard_PresetCell::~ExtensionMidi_GlobalKeyboard_PresetCell()
{
    delete ui;
}

void ExtensionMidi_GlobalKeyboard_PresetCell::on_check16_stateChanged(int arg1)
{
    Q_UNUSED(arg1)
    storeState();
}


void ExtensionMidi_GlobalKeyboard_PresetCell::on_check8_stateChanged(int arg1)
{
    Q_UNUSED(arg1)
    storeState();
}


void ExtensionMidi_GlobalKeyboard_PresetCell::on_check4_stateChanged(int arg1)
{
    Q_UNUSED(arg1)
    storeState();
}

void ExtensionMidi_GlobalKeyboard_PresetCell::storeState()
{
    quint8 state = (ui->check8->isChecked() ? 0x01 : 0) +
            (ui->check4->isChecked() ? 0x02 : 0) +
            (ui->check16->isChecked() ? 0x04 : 0);
    _extensionConfig->setActivationState(_channel, _indexSf2, _indexElt, state);
}

void ExtensionMidi_GlobalKeyboard_PresetCell::clear()
{
    ui->check4->setChecked(false);
    ui->check8->setChecked(false);
    ui->check16->setChecked(false);
}
