/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#ifndef EXTENSIONVIEW_BARS_GRAPHICS_H
#define EXTENSIONVIEW_BARS_GRAPHICS_H

#include <QGraphicsView>
#include "basetypes.h"
class SoundfontManager;
class ExtensionView_Bars_Graphics_LegendItem;
class ExtensionView_Bars_Graphics_LegendItem2;
class ExtensionView_Bars_Graphics_RectangleItem;
class ExtensionView_Bars_Graphics_Key;
class GraphicsSimpleTextItem;

class ExtensionView_Bars_Graphics : public QGraphicsView
{
    Q_OBJECT

public:
    explicit ExtensionView_Bars_Graphics(QWidget *parent = nullptr);
    ~ExtensionView_Bars_Graphics();

    void display(IdList ids, AttributeType attributeType, bool justSelection);
    void playKey(int key, int velocity);

    static EltID findBrother(EltID id);

signals:
    void selectionChanged(IdList selectedIds);

protected:
    void showEvent(QShowEvent * event) override;
    void resizeEvent(QResizeEvent * event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseDoubleClickEvent(QMouseEvent *) override;
    void keyPressEvent(QKeyEvent *event) override;
    void keyReleaseEvent(QKeyEvent *event) override;
    void wheelEvent(QWheelEvent * event) override;

private:
    enum Mode
    {
        NORMAL_MODE,
        EDITING_MODE, // Left mouse click held
        EDITING_MODE_WITH_MOVE, // Left mouse click held + something edited
        CONTOUR_MODE, // Alt key held
        EDITING_CONTOUR_MODE, // Alt key + left mouse click held
        PLAY_MODE // Middle mouse click held
    };

    void initItems();
    void updateLabelPosition();
    void updateHover(QPoint mousePos);
    void updateHorizontalLines(AttributeType attributeType, bool isPrst);
    QList<QList<ExtensionView_Bars_Graphics_RectangleItem *> > getRectanglesUnderMouse(QPoint mousePos);
    void triggerDivisionSelected();
    QList<ExtensionView_Bars_Graphics_RectangleItem *> getNextUnselectedRectangles(bool toTheRight);
    void setHeightForRectanglesBelowMouse(QPoint mousePos);

    SoundfontManager * _sf2;
    EltID _defaultID;

    // Graphics items
    QGraphicsScene * _scene;
    QList<ExtensionView_Bars_Graphics_RectangleItem *> _rectangles;
    ExtensionView_Bars_Graphics_LegendItem * _legendItem;
    ExtensionView_Bars_Graphics_LegendItem2 * _legendItem2;
    QList<GraphicsSimpleTextItem *> _leftLabels, _bottomLabels;
    QList<QGraphicsLineItem *> _vLines, _hLines;
    QMap<int, ExtensionView_Bars_Graphics_Key*> _mapGraphicsKeys;

    // Various
    int _keyTriggered;
    bool _removeIndexOnRelease, _removeOtherIndexOnRelease;
    Mode _currentMode;
    bool _keepFirstShiftPoint;
    double _yInit;
    int _minKey, _maxKey;
    QRectF _displayedRect;
    QRectF _firstShiftRect;

    // Configuration
    QColor _textColor, _lineColor;
    static const int HLINE_NUMBER;
};

#endif // EXTENSIONVIEW_BARS_GRAPHICS_H
