/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "extensionview_bars_graphics_key.h"
#include <QPainter>
#include <QTimer>
#include <QApplication>
#include <QPalette>
#include "contextmanager.h"

const double ExtensionView_Bars_Graphics_Key::s_width = 0.9;
const int ExtensionView_Bars_Graphics_Key::s_maxAlpha = 60;

ExtensionView_Bars_Graphics_Key::ExtensionView_Bars_Graphics_Key(QGraphicsItem *parent) : QObject(nullptr), QGraphicsItem(parent)
{
    // Color
    _color = ContextManager::theme()->getColor(ThemeManager::LIST_TEXT);
    _color.setAlpha(4);

    QTimer::singleShot(30, this, SLOT(updateColor()));
}

QRectF ExtensionView_Bars_Graphics_Key::boundingRect() const
{
    return QRectF(-s_width / 2 + _key, -100, s_width, 200);
}

void ExtensionView_Bars_Graphics_Key::paint(QPainter *painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
    Q_UNUSED(option)
    Q_UNUSED(widget)

    painter->setPen(QPen(_color, s_width));
    painter->drawLine(_key, -100, _key, 100);
}

void ExtensionView_Bars_Graphics_Key::updateColor()
{
    _color.setAlpha(qMin(255., 25 + 1.1 * _color.alpha()));
    if (_color.alpha() < s_maxAlpha)
        QTimer::singleShot(30, this, SLOT(updateColor()));

    this->update();
}
