/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2020 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone-soundfonts.com                 **
**             Date: 01.01.2013                                           **
***************************************************************************/

#ifndef EXTENSIONVIEW_BARS_GRAPHICS_RECTANGLEITEM_H
#define EXTENSIONVIEW_BARS_GRAPHICS_RECTANGLEITEM_H

#include <QGraphicsItem>
#include <QPen>
#include "basetypes.h"

class ExtensionView_Bars_Graphics_RectangleItem : public QGraphicsRectItem
{
public:
    enum StepType
    {
        SMALL_STEP,
        MIDDLE_STEP,
        BIG_STEP
    };

    ExtensionView_Bars_Graphics_RectangleItem(EltID id, AttributeType attributeType, QGraphicsItem *parent = nullptr);

    EltID getID() { return _id; }

    void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = nullptr) override;
    bool contains(const QPointF &point) const override;

    void setNewValue(double delta);
    void addStep(StepType stepType, bool isPositive);
    void setPercent(float percent /* between 0 and 1 */);

    bool saveChanges(); // Return true if changes have been made
    void cancelChanges();

    void setHover(bool isHovered);
    bool isHovered() { return _editingMode; }
    static void syncHover(bool isSync) { s_synchronizeEditingMode = isSync; }
    void setSelected(bool isSelected) { _isSelected = isSelected; }
    bool isSelected() { return _isSelected; }

    bool operator==(const ExtensionView_Bars_Graphics_RectangleItem& other) { return (_id == other._id); }
    bool operator==(const EltID &id) { return _id == id; }

    int currentMinKey() { return _minKey; }
    int currentMaxKey() { return _maxKey; }
    double currentVal() { return _currentVal; }

    static double getNormalizedValue(double value, AttributeType attributeType, bool isPrst);
    static double getValue(double normalizedValue, AttributeType attributeType, bool isPrst);

private:
    void initialize(EltID id, AttributeType attributeType);
    QRectF getRectF() const;

    QPen _penBorderThin;
    QPen _penBorderFat;
    QBrush _brushRectangle;
    QBrush _brushRectangleSelected;

    EltID _id;
    AttributeType _attributeType;
    int _minKey, _maxKey;
    double _currentValInit;
    double _currentVal;
    bool _editingMode;
    bool _isSelected;
    static bool s_synchronizeEditingMode;
    static const double BAR_THICKNESS;
};

#endif // EXTENSIONVIEW_BARS_GRAPHICS_RECTANGLEITEM_H
